==> dmc-sdk-4.0.0.0-sources.jar: com/digiwin/dmc/sdk/service/upload/CoverSegmentDocumentUploader.java <==
package com.digiwin.dmc.sdk.service.upload;

import com.digiwin.dmc.sdk.config.DmcUrl;
import com.digiwin.dmc.sdk.config.NetworkOptions;
import com.digiwin.dmc.sdk.config.SDKConstants;
import com.digiwin.dmc.sdk.config.ServerSetting;
import com.digiwin.dmc.sdk.entity.FileInfo;
import com.digiwin.dmc.sdk.exception.OperateException;
import com.digiwin.dmc.sdk.service.IUserManagerService;
import com.digiwin.dmc.sdk.service.impl.UserManagerService;
import com.digiwin.dmc.sdk.util.HttpRequestUtil;
import com.digiwin.dmc.sdk.util.ObjectMapperUtil;
import com.digiwin.dmc.sdk.util.StringUtil;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class CoverSegmentDocumentUploader implements ICoverSegmentDocumentUploader {

    private String tenantId;
    private String bucketName;
    private int bufferSize;
    private String driveToken;
    private UploadProcessState state;
    private String fileId;
    private FileInputStream fileInputStream;
    private InputStream inputStream;
    private IUpLoadCallbackable progressChangedCallback;
    private IUpLoadCallbackable completeCallback;
    private UploadProgressEventArgs eventArgs = new UploadProgressEventArgs();


    private static ExecutorService es = Executors.newCachedThreadPool();
    private static IUserManagerService userManagerService = UserManagerService.userInstance();

    public CoverSegmentDocumentUploader(String tenantId, String bucketName, String localPath, String fileId, String driveToken) {
        try {
            File file = new File(localPath);
            FileInputStream fileInputStream = new FileInputStream(file);
            this.fileInputStream = fileInputStream;
            eventArgs.setTotalBytes((int) file.length());
        } catch (Exception e) {
            throw new OperateException(e);
        }
        this.driveToken = driveToken;
        this.bucketName = bucketName;
        this.tenantId = tenantId;
        this.fileId = fileId;
        eventArgs.setPercentage(0);
        eventArgs.setCompletedBytes(0);
        this.state = UploadProcessState.NotStarted;
        this.bufferSize = NetworkOptions.Default.getPackageSize();
    }

    public CoverSegmentDocumentUploader(String tenantId, String bucketName, byte[] bytes, String fileId, String driveToken) {
        try {
            this.inputStream= new ByteArrayInputStream(bytes);
            eventArgs.setTotalBytes(inputStream.available());
        } catch (Exception e) {
            throw new OperateException(e);
        }
        this.driveToken = driveToken;
        this.bucketName = bucketName;
        this.tenantId = tenantId;
        this.fileId = fileId;
        eventArgs.setPercentage(0);
        eventArgs.setCompletedBytes(0);
        this.state = UploadProcessState.NotStarted;
        this.bufferSize = NetworkOptions.Default.getPackageSize();
    }

    /**
     * 异步上传Stream中的文件内容
     *
     * @return 上传文件Id
     */
    @Override
    public ICoverSegmentDocumentUploader beginUpload() {

        if (state != UploadProcessState.NotStarted) {
            throw new OperateException("上传已开始，无法再次启动下载，请开启新的上传");
        }
        if (StringUtil.isEmpty(bucketName)) {
            bucketName = ServerSetting.getBucketName();
        }
        this.continueUpload();
        Runnable runnable = () -> {
            byte[] buffer = new byte[bufferSize];
            int bytesReaded = 0;
            int totalBytes = eventArgs.getTotalBytes();

            while (bytesReaded < totalBytes) {
                try {
                    bytesReaded = eventArgs.getCompletedBytes();
                    if (state == UploadProcessState.Stopped) {
                        break;
                    } else if (state == UploadProcessState.Paused) {
                        Thread.sleep(1000 * 60 * 60 * 24);
                        if (state == UploadProcessState.Paused) {
                            state = UploadProcessState.Timeout;
                            break;
                        }
                    } else {
                        int currentReadLen = 0;
                        currentReadLen = fileInputStream.read(buffer, 0, bufferSize);
                        if (currentReadLen == -1) {
                            break;
                        }
                        if (bytesReaded + bufferSize >= totalBytes) {
                            currentReadLen = totalBytes - bytesReaded;
                            this.uploadSegFile(bucketName, bytesReaded, currentReadLen, buffer);
                            eventArgs.setCompletedBytes(totalBytes);
                            eventArgs.setPercentage(1);
                            if (progressChangedCallback != null) {
                                progressChangedCallback.callback(eventArgs);
                            }
                            if (completeCallback != null) {
                                eventArgs.setFileId(fileId);
                                completeCallback.callback(eventArgs);
                            }
                            return;
                        } else {
                            this.uploadSegFile(bucketName, bytesReaded, currentReadLen, buffer);
                            eventArgs.setCompletedBytes(bytesReaded + currentReadLen);
                            eventArgs.setPercentage((double) eventArgs.getCompletedBytes() / (double) eventArgs.getTotalBytes());
                            if (progressChangedCallback != null) {
                                progressChangedCallback.callback(eventArgs);
                            }
                        }
                    }
                } catch (Exception e) {
                    throw new OperateException(e);
                }
            }
        };
        es.execute(runnable);
        return this;

    }

    public void uploadSegFile(String bucketName, int from, int count, byte[] buf) {
        if (StringUtil.isEmpty(bucketName)) {
            bucketName = ServerSetting.getBucketName();
        }
        String uploadResourceUrl = DmcUrl.getFilesResourceUrl() + bucketName +
                "/files/" + fileId + "/" + from + "/" + (from + count - 1) + "/" + this.eventArgs.getTotalBytes() + "/cover";
        Map<String, String> headers = new HashMap<>();
        if (driveToken == null || "".equals(driveToken)) {
            String userToken = userManagerService.getUserToken(ServerSetting.getUser());
            headers.put(SDKConstants.HttpHeaderUserTokenKey, userToken);
        } else {
            headers.put(SDKConstants.HttpHeaderDriveTokenKey, driveToken);
        }
        if (!StringUtil.isEmpty(tenantId)) {
            headers.put(SDKConstants.HTTP_HEADER_TENANTID_KEY, tenantId);
        }
        HttpRequestUtil.uploadSegFile(uploadResourceUrl, headers, count, buf);
    }

    /**
     * 设置响应进度
     *
     * @param callbackable
     */
    @Override
    public ICoverSegmentDocumentUploader onProgressChanged(IUpLoadCallbackable callbackable) {
        this.progressChangedCallback = callbackable;
        return this;
    }

    /**
     * 设置完成
     *
     * @param callbackable
     */
    @Override
    public ICoverSegmentDocumentUploader onCompleted(IUpLoadCallbackable callbackable) {
        this.completeCallback = callbackable;
        return this;
    }

    /**
     * 暂停上传
     */
    @Override
    public void pauseUpload() {
        state = UploadProcessState.Paused;
    }

    /**
     * 继续上传
     */
    @Override
    public void continueUpload() {
        state = UploadProcessState.Uploading;
    }

    /**
     * 停止上传
     */
    @Override
    public void stopUpload() {
        state = UploadProcessState.Stopped;
    }

}

==> dmc-sdk-5.2.2.0-sources.jar: com/digiwin/dmc/sdk/service/upload/CoverSegmentDocumentUploader.java <==
package com.digiwin.dmc.sdk.service.upload;

import com.digiwin.dap.middleware.dmc.common.context.TenantId;
import com.digiwin.dmc.sdk.config.NetworkOptions;
import com.digiwin.dmc.sdk.config.ServerSetting;
import com.digiwin.dmc.sdk.exception.OperateException;
import com.digiwin.dmc.sdk.util.ArgumentUtils;
import com.digiwin.dmc.sdk.util.ExecutorUtil;
import com.digiwin.dmc.sdk.util.HttpRequestUtil;
import com.digiwin.dmc.sdk.util.HttpUtils;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Map;

/**
 * 覆盖分段上传
 * /api/dmc/v1/buckets/{bucket}/files/{fileId}/{from}/{to}/{total}/cover
 *
 * @date 2022/6/7 14:12
 */
public class CoverSegmentDocumentUploader implements ICoverSegmentDocumentUploader {

    private String bucketName;
    private String fileId;
    private String driveToken;
    private TenantId[] tenantId;
    private int bufferSize = NetworkOptions.Default.getPackageSize();
    private InputStream inputStream;
    private IUpLoadCallbackable completeCallback;
    private IUpLoadCallbackable progressChangedCallback;
    private UploadProcessState state = UploadProcessState.NotStarted;
    private UploadProgressEventArgs eventArgs = new UploadProgressEventArgs();

    private CoverSegmentDocumentUploader(String bucketName, String fileId, String driveToken, TenantId... tenantId) {
        this.bucketName = bucketName;
        this.fileId = fileId;
        this.driveToken = driveToken;
        this.tenantId = tenantId;
    }

    public CoverSegmentDocumentUploader(String bucketName, String localPath, String fileId, String driveToken, TenantId... tenantId) {
        this(bucketName, fileId, driveToken, tenantId);
        try {
            this.inputStream = Files.newInputStream(Paths.get(localPath));
            eventArgs.setTotalBytes(inputStream.available());
        } catch (IOException e) {
            throw new OperateException(e);
        }
        eventArgs.setPercentage(0);
        eventArgs.setCompletedBytes(0);
    }

    public CoverSegmentDocumentUploader(String bucketName, byte[] bytes, String fileId, String driveToken, TenantId... tenantId) {
        this(bucketName, fileId, driveToken, tenantId);
        try {
            this.inputStream = new ByteArrayInputStream(bytes);
            eventArgs.setTotalBytes(inputStream.available());
        } catch (IOException e) {
            throw new OperateException(e);
        }
        eventArgs.setPercentage(0);
        eventArgs.setCompletedBytes(0);
    }

    /**
     * 异步上传Stream中的文件内容
     *
     * @return 上传文件Id
     */
    @Override
    public ICoverSegmentDocumentUploader beginUpload() {
        if (state != UploadProcessState.NotStarted) {
            throw new OperateException("上传已开始，无法再次启动下载，请开启新的上传");
        }
        bucketName = ArgumentUtils.getBucketName(bucketName);

        this.continueUpload();

        Map<String, String> headers = HttpUtils.setHeader(driveToken, tenantId);

        Runnable runnable = () -> {
            byte[] buffer = new byte[bufferSize];
            int bytesReaded = 0;
            int totalBytes = eventArgs.getTotalBytes();

            while (bytesReaded < totalBytes) {
                try {
                    bytesReaded = eventArgs.getCompletedBytes();
                    if (state == UploadProcessState.Stopped) {
                        break;
                    } else if (state == UploadProcessState.Paused) {
                        Thread.sleep(1000 * 60 * 60 * 24);
                        if (state == UploadProcessState.Paused) {
                            state = UploadProcessState.Timeout;
                            break;
                        }
                    } else {
                        int currentReadLen = inputStream.read(buffer, 0, bufferSize);
                        if (currentReadLen == -1) {
                            break;
                        }
                        if (bytesReaded + bufferSize >= totalBytes) {
                            currentReadLen = totalBytes - bytesReaded;
                            this.uploadSegFile(bucketName, headers, bytesReaded, currentReadLen, buffer);
                            eventArgs.setCompletedBytes(totalBytes);
                            eventArgs.setPercentage(1);
                            if (progressChangedCallback != null) {
                                progressChangedCallback.callback(eventArgs);
                            }
                            if (completeCallback != null) {
                                eventArgs.setFileId(fileId);
                                completeCallback.callback(eventArgs);
                            }
                            return;
                        } else {
                            this.uploadSegFile(bucketName, headers, bytesReaded, currentReadLen, buffer);
                            eventArgs.setCompletedBytes(bytesReaded + currentReadLen);
                            eventArgs.setPercentage((double) eventArgs.getCompletedBytes() / (double) eventArgs.getTotalBytes());
                            if (progressChangedCallback != null) {
                                progressChangedCallback.callback(eventArgs);
                            }
                        }
                    }
                } catch (Exception e) {
                    throw new OperateException(e);
                }
            }
        };
        ExecutorUtil.newExecutor().execute(runnable);
        return this;

    }

    public void uploadSegFile(String bucketName, Map<String, String> headers, int from, int count, byte[] buf) {
        String uri = String.format("%s/api/dmc/v1/buckets/%s/files/%s/%s/%s/%s/cover",
                ServerSetting.getServiceUrl(), bucketName, fileId, from, (from + count - 1), this.eventArgs.getTotalBytes());
        HttpRequestUtil.uploadSegFile(uri, headers, count, buf);
    }

    /**
     * 设置响应进度
     *
     * @param callbackable
     */
    @Override
    public ICoverSegmentDocumentUploader onProgressChanged(IUpLoadCallbackable callbackable) {
        this.progressChangedCallback = callbackable;
        return this;
    }

    /**
     * 设置完成
     *
     * @param callbackable
     */
    @Override
    public ICoverSegmentDocumentUploader onCompleted(IUpLoadCallbackable callbackable) {
        this.completeCallback = callbackable;
        return this;
    }

    /**
     * 暂停上传
     */
    @Override
    public void pauseUpload() {
        state = UploadProcessState.Paused;
    }

    /**
     * 继续上传
     */
    @Override
    public void continueUpload() {
        state = UploadProcessState.Uploading;
    }

    /**
     * 停止上传
     */
    @Override
    public void stopUpload() {
        state = UploadProcessState.Stopped;
    }

}
